# Amahi Home Server
# Copyright (C) 2007-2010 Amahi
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License v3
# (29 June 2007), as published in the COPYING file.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# file COPYING for more details.
# 
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Amahi
# team at http://www.amahi.org/ under "Contact Us."

# Filters added to this controller apply to all controllers in the application.
# Likewise, all the methods added will be available for all controllers.

class ApplicationController < ActionController::Base

	helper :all # include all helpers, all the time
	protect_from_forgery

	before_filter :before_filter_hook
	filter_parameter_logging :password, :passwd, :passwd_confirm

	helper_method :current_user

	layout 'amahi'

	VALID_DHCP_ADDRESS_RANGE = 100

	def before_filter_hook
		set_locale
		check_for_amahi_app
		prepare_theme
		adv = Setting.find_by_name('advanced')
		@advanced = adv && adv.value == '1'
	end

	def check_for_amahi_app
		server = request.env['SERVER_NAME']
		dom = Setting.get('domain')
		if server && server != 'hda' && server =~ /^(.*)\.#{dom}$/
			server = $1
		end
		if server && server != 'hda' && Alias.find_by_alias(server)
			redirect_to "http://hda/hda_app_#{server}"
		end
	end

	def prepare_theme
		@app_headers = []
		@theme_name = 'main'
		@theme_gruff = {}
		@theme_headers = []
		ts = Setting.find(:first, :conditions => "name = 'theme'")
		if ts.nil?
		  Setting.create!(:name => 'theme', :value => @theme_name);
		else
		    @theme_name = ts.value || 'main'
		end
		theme_init_file = "#{RAILS_ROOT}/public/themes/#{@theme_name}/init.rb"
		unless File.exist? theme_init_file
		    @theme_name = 'main' unless File.exist? theme_init_file
		    theme_init_file = "#{RAILS_ROOT}/public/themes/#{@theme_name}/init.rb"
		end
		if File.exist? theme_init_file
		    load theme_init_file
		    if defined?(theme_init) == "method"
		    	begin
		    		theme = theme_init
		    		@theme_gruff = theme[:gruff_theme]
		    		@theme_author = theme[:author_name]
		    		@theme_url = theme[:author_url]
				@theme_disable_inheritance = theme[:disable_inheritance]
		    		@theme_headers = theme[:headers].map{|h| (h =~ /\.js$/) ? "/themes/#{@theme_name}/#{h}" : h } if theme[:headers]
			# be lenient about theme issues!
		    	rescue => e
		    		# there were issues in the theme init file!!
		    		logger.error("=======================================================================")
		    		logger.error("=================== Amahi Theme Error BEGIN ===========================")
		    		logger.error(e)
		    		logger.error("=================== Amahi Theme Error END   ===========================")
		    		logger.error("=======================================================================")
		    	end
		    end
		end
		# session = Hash.new unless session
		session[:theme] = @theme_name
	end


	class Helper
		include Singleton
		include ActionView::Helpers::NumberHelper
	end

	def number_helpers
		Helper.instance
	end

	def setup_router
		@router = nil
		r = Setting.get_kind('network', 'router_model')
		return @router unless r
		begin
			rd = RouterDriver.current_router = (r ? r.value : "")
			# return the class proper if valid
			@router = Kernel.const_get(rd) unless rd.blank?
			u = Setting.network.find_by_name('router_username')
			p = Setting.network.find_by_name('router_password')
			RouterDriver.set_auth(unobfuscate(u.value), unobfuscate(p.value)) if p and u and p.value and u.value
		rescue
			# shhh. comment out the rescue for debugging
		end
		@router
	end

	def locales_implemented
		# to add new languages, please visit this page
		#         http://en.wikipedia.org/wiki/List_of_ISO_639-1_codes
		# and pick columns 6 (name of the language *in the native language*) and column 1 (ISO code)
		# Please put them here in alphabetical order so that the user can more easily find his/hers!
		[['العربية', 'ar'], ['Bahasa Indonesia', 'id'], ['български език', 'bg'], ['Dansk', 'da'], ['Deutsch', 'de'],
		['Ελληνικά', 'el'], ['English', 'en'], ['Español', 'es'], ['Français', 'fr'],
		['עברית', 'he'], ['hrvatski', 'hr'], ["latviešu valoda", 'lv'],
		['Italiano', 'it'], ['日本語', 'ja'], ['한국어', 'ko'], ['Nederlands', 'nl'],
		['Norsk (Bokmål)', 'no-NB'], ['Norsk (Nynorsk)', 'no-NN'], ['Polski', 'pl'],
		['Português - Brasil', 'pt-br'], ['Português - Portugal', 'pt-pt'], ['română', 'ro'],
		['Русский язык', 'ru'], ['suomi', 'fi'], ['Svenska', 'sv'], ['中文', 'zh']]
	end

private

	def set_locale
		preferred_locales = request.headers['HTTP_ACCEPT_LANGUAGE'].split(',').map { |l| l.split(';').first } rescue nil
		I18n.locale = begin
			locale = preferred_locales.select { |l| I18n.available_locales.include?(l.to_sym) }
			default_locale = locale.empty? ? I18n.default_locale : locale.first

			# Allow a URL param to override everything else, for devel
			if params[:locale]
				if I18n.available_locales.include?(params[:locale].to_sym)
					cookies['locale'] = { :value => params[:locale], :expires => 1.year.from_now }
					params[:locale].to_sym
				else
					cookies.delete 'locale'
					default_locale
				end
			elsif cookies['locale'] && I18n.available_locales.include?(cookies['locale'].to_sym)
				cookies['locale'].to_sym
			else
				cookies['locale'] = { :value => default_locale, :expires => 1.year.from_now }
				default_locale
			end
		rescue
			# if something happens (like a locale file renamed!?) go back to the default
			I18n.default_locale
		end

		# right to left language support
		if [:he, :ar].include? I18n.locale
			@locale_direction = "rtl"
		else
			@locale_direction = "ltr"
		end
	end

	# FIXME: these are simple rot13
	def obfuscate(s)
		s.tr("A-Ma-mN-Zn-z","N-Zn-zA-Ma-m")
	end

	def unobfuscate(s)
		s.tr("N-Zn-zA-Ma-m", "A-Ma-mN-Zn-z")
	end

	def current_user_session
		return @current_user_session if defined?(@current_user_session)
		@current_user_session = UserSession.find
	end

	def current_user
		return @current_user if defined?(@current_user)
		@current_user = current_user_session && current_user_session.record
	end

	def login_required
		unless current_user
			store_location
			flash[:notice] = t('must_be_logged_in')
			redirect_to new_user_session_url
			return false
		end
	end

	def login_required_unless_guest_dashboard
		guest_dashboard = Setting.get("guest-dashboard")
		return true if guest_dashboard && guest_dashboard == "1"
		login_required
	end

	def admin_required
		return false if login_required == false
		unless current_user.admin?
			store_location
			flash[:notice] = t('must_be_admin')
			redirect_to new_user_session_url
			return false
		end
	end

	def store_location
		session[:return_to] = request.request_uri
	end
end
