# Amahi Home Server
# Copyright (C) 2007-2010 Amahi
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License v3
# (29 June 2007), as published in the COPYING file.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# file COPYING for more details.
# 
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Amahi
# team at http://www.amahi.org/ under "Contact Us."

class UserController < ApplicationController
	before_filter :admin_required

	# FIXME - 746 and 749 to allow other charsets!
	VALID_USERNAME = Regexp.new "^[A-Za-z][A-Za-z0-9]+$"

	def index
		@users = User.all_users
	end

	def delete
		u = User.find params[:id]
		if u and !u.admin and u != current_user
			u.destroy
		end
		users = User.find :all
		render :partial => 'user/list', :locals => { :users => users }
	end

	def create
		fn = params[:full_name]
		un = params[:user_name]
		pwd = params[:passwd]
		conf = params[:passwd_confirm]
		if un.nil? or un.blank? or not valid_name?(un)
			flash.now[:error] = "Bad user name!"
			render :text => "", :status => 403
			return
		end
		if fn.nil? or fn.blank?
			render :text => "", :status => 401
			return
		end
		if pwd.nil? or (pwd.size < 2) or (pwd != conf)
			render :text => "", :status => 403
			return
		end
		u = User.new(:name => fn, :login => un, :password => pwd, :password_confirmation => conf)
		u.save!
		u.reload
		users = User.all_users
		render :partial => "user/body", :locals => { :users => users }
	end

	def update_name
		id = params[:id]
		u = User.find(id)
		unless params[:value].blank?
			name = params[:value]
			u.name = name
			u.save
			u.reload
		end
		name = u.name
		render :text => name
	end


	def new_user_username_check
		un = params[:user_name]
		if un.nil? or un.blank?
			render :partial => 'user/name_cannot_be_blank'
			return false
		end
		unless valid_name?(un)
			render :partial => 'user/name_invalid'
			return false
		end
		user = User.find_by_login(un)
		if user
			render :partial => 'user/name_not_available'
			return false
		else
			render :partial => 'user/name_available'
			return true
		end
	end

	def new_user_fullname_check
		un = params[:full_name]
		if un.nil? or un.blank?
			render :partial => 'user/fullname_cannot_be_blank'
			return false
		end
		render :partial => 'user/fullname_clear'
		return true
	end

	def new_user_password_check
		pwd = params[:passwd]
		if pwd.nil? or pwd.blank?
			render :partial => 'user/password_cannot_be_blank'
			return false
		end
		if pwd.size < 5
			render :partial => 'user/password_warn_too_short'
			return false
		end
		render :partial => 'user/password_clear'
		return true
	end

	def new_user_password_confirm_check
		pwd = params[:passwd]
		conf = params[:passwd_confirm]
		if pwd.nil? or pwd.blank?
			render :partial => 'user/confirmation_must_match'
			return false
		end
		if conf.size > 1 and pwd == conf
			render :partial => 'user/confirmation_is_good'
			return true
		end
		render :partial => 'user/confirmation_must_match'
		return false
	end

	def password_change_block
		render :partial => 'user/password_change_block', :locals => { :user => params[:id] }
	end

	def password_change
		pwd = params[:passwd]
		conf = params[:passwd_confirm]
		raise "bad confirmation/password" unless valid_password?(pwd, conf)
		u = User.find params[:id]
		u.update_attributes(:password => pwd, :password_confirmation => conf)
		u.add_or_passwd_change_samba_user
		render :partial => 'user/password_change', :locals => { :uid => params[:id] }
	end

	def toggle_admin
		user = User.find params[:id]
		if current_user == user or user.needs_auth?
			# cannot upgrade or downgrade itself, nor change users that need auth
			render :partial => 'user/is_admin', :locals => { :user => user }
			return
		end
		user.admin = !user.admin
		user.save!
		render :partial => 'user/is_admin', :locals => { :user => user }
	end

protected

	def valid_password?(pwd, conf)
		return false if pwd.nil? or pwd.blank?
		return true if conf.size > 1 and pwd == conf
		false
	end

	def valid_name?(nm)
		return false if nm.size < 3
		return false if nm.size > 32
		return false unless (nm =~ VALID_USERNAME)
		return false unless User.is_valid_name?(nm)
		true
	end

	# FIXME: too simple password checking!
	# note - some users may even want less restrictions!
	def passwd_check(pwd, conf)
	      (not pwd.nil?) and pwd == conf and pwd.size > 1
	end

end
