# Amahi Home Server
# Copyright (C) 2007-2010 Amahi
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License v3
# (29 June 2007), as published in the COPYING file.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# file COPYING for more details.
# 
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Amahi
# team at http://www.amahi.org/ under "Contact Us."

require 'uri'

# Methods added to this helper will be available to all templates in the application.
module ApplicationHelper
	DEFAULT_THEME_NAME = 'main'

	def theme_stylesheet_link_tag(source, fname='style')
	    source = theme_stylesheet_path(source, fname)
	    tag("link", { "rel" => "Stylesheet", "type" => "text/css", "media" => "screen", "href" => source })
	end

	def is_a_mac?
		(request.env["HTTP_USER_AGENT"] =~ /Macintosh/) ? true : false
	end

	def is_firefox?
		(request.env["HTTP_USER_AGENT"] =~ /Firefox/) ? true : false
	end

	def theme_stylesheet_path(theme, fname='style')
	      compute_public_path(fname, File.join('themes', theme), 'css')
	      # FIXME - why doesn't this work??? it always says the
	      # file is not there and defaults to the main theme!
	      # but the file is there when checked by hand - plus this
	      # works on theme_image_path!
	      #
	      #	if File.exists?(File.join(RAILS_ROOT, 'public', path))
	      #		path
	      #	else
	      #		# default to the default theme - safe
	      #		compute_public_path("style", File.join('themes', DEFAULT_THEME_NAME), 'css')
	      #	end
	end

	def theme_image_tag(source, options = {})
	  options.symbolize_keys!
	          
	  options[:src] = theme_image_path(source)
	  options[:alt] ||= File.basename(options[:src], '.*').split('.').first.capitalize
	  options[:title] ||= options[:alt]
	  options[:class] ||= "theme-image"
	  
	  if options[:size]
	    options[:width], options[:height] = options[:size].split("x") if options[:size] =~ %r{^\d+x\d+$}
	    options.delete(:size)
	  end

	  tag("img", options)
	end

	def theme_image_path(source)
	      source += '.png' if File.extname(source).blank?
	      path = File.join('themes', @theme_name, source)
	      if File.exists?(File.join(RAILS_ROOT, 'public', path))
	      	compute_public_path(source, File.join('themes', @theme_name), 'png')
	      else
	      	# default to the default theme - safe
	      	compute_public_path(source, File.join('themes', DEFAULT_THEME_NAME), 'png')
	      end
	end

	def theme_named_image_tag(source, theme = @theme_name, options = {})
	  options.symbolize_keys!
	          
	  options[:src] = theme_named_image_path(source, theme)
	  options[:alt] ||= File.basename(options[:src], '.*').split('.').first.capitalize
	  
	  if options[:size]
	    options[:width], options[:height] = options[:size].split("x") if options[:size] =~ %r{^\d+x\d+$}
	    options.delete(:size)
	  end

	  tag("img", options)
	end

	def theme_named_image_path(source, theme)
	  compute_public_path(source, File.join('themes', theme), 'png')
	end

	def editable_content(options)
		options[:content] = { :element => 'span' }.merge(options[:content])
		options[:url] = {}.merge(options[:url])
		options[:ajax] = { :okText => "'Save'", :cancelText => "'Cancel'"}.merge(options[:ajax] || {})
		script = Array.new
		script << "new Ajax.InPlaceEditor("
		script << "  '#{options[:content][:options][:id]}',"
		script << "  '#{url_for(options[:url])}',"
		script << "  {"
		script << options[:ajax].map{ |key, value| "#{key.to_s}: #{value}" }.join(", ")
		script << "  }"
		script << ")"

		content_tag(
		  options[:content][:element],
		  options[:content][:text],
		  options[:content][:options]
		) + javascript_tag( script.join("\n") )
	end

	# FIXME-cpg: somehow generate <input ... /> instead of
	# <input>...</input>
	def checkbox_to_function(checked = true, *args, &block)
		html_options = args.extract_options!
		function = args[0] || ''

		html_options.symbolize_keys!
		function = update_page(&block) if block_given?
		tag("input",
		  html_options.merge({ 
		    :type => "checkbox",
		    :checked => ("checked" if checked),
		    # FIXME-cpg: href should not be needed? :href => html_options[:href] || "#", 
		    :onclick => (html_options[:onclick] ? "#{html_options[:onclick]}; " : "") + "#{function}; return false;" 
		  })
		)
	end

	def checkbox_to_remote( checked = true, options = {}, html_options = nil)
		checkbox_to_function(checked, remote_function(options), html_options || options.delete(:html))
	end

	def path2uri(name)
		name = URI.escape name
		is_a_mac? ? "smb://hda/#{name}" : "file://///hda/#{name}"
	end

	def path2location(name)
		fwd = '\\'
		is_a_mac? ? '&raquo; ' + h(name.gsub(/\//, ' ▸ ')) : h('\\\\hda\\' + name.gsub(/\//, fwd))
	end

	def fw_rule_type(t)
		case t
		when 'port_filter'
			'Port Filter'
		when 'url_filter'
			'URL Filter'
		when 'mac_filter'
			'MAC Filter'
		when 'ip_filter'
			'IP Filter'
		when 'port_forward'
			'Port Forwarding'
		else
			raise "type #{rule.kind} unknown"
		end
	end

	def fw_rule_details(rule)
		case rule.kind
		when 'port_filter'
			"Ports: #{rule.range}, Protocol: #{fw_prot(rule.protocol)}"
		when 'ip_filter'
			"IP: #{@net}.#{rule.ip}, Protocol: #{fw_prot(rule.protocol)}"
		when 'mac_filter'
			"MAC: #{rule.mac}"
		when 'url_filter'
			"URL: #{rule.url}"
		when 'port_forward'
			"IP: #{@net}.#{rule.ip}, Ports: #{rule.range}"
		else
			raise "details for #{rule.kind} unknown"
		end
	end

	def fw_rule_state(rule)
		Setting.get(rule.kind) == '1'
	end

	def fw_prot(p)
		p == 'both' ? 'TCP &amp; UDP' : p.upcase
	end

	def msg_bad(s = "")
		theme_image_tag("stop") + " " + s
	end

	def msg_good(s = "")
		theme_image_tag("ok") + " " + s
	end

	def msg_warn(s = "")
		theme_image_tag("warning") + " " + s
	end

	def delete_icon(title = "")
		theme_image_tag("delete.png", :title => title)
	end

	def inline_event
		page << "new Effect.Event({queue: 'end', afterFinish:function(){"
		yield
		page << "}})"
	end

	def spinner id
		theme_image_tag("working.gif", :id => "spinner-#{id}", :style=> "display: none;")
	end

	def spinner_show id
		"Element.show('spinner-#{id}');"
	end

	def spinner_hide id
		"Element.hide('spinner-#{id}');"
	end
end
