# Amahi Home Server
# Copyright (C) 2007-2010 Amahi
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License v3
# (29 June 2007), as published in the COPYING file.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# file COPYING for more details.
# 
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Amahi
# team at http://www.amahi.org/ under "Contact Us."

require 'strscan'
require 'command'

class User < ActiveRecord::Base

	named_scope :admins, :conditions => "admin = 1"

	begin
		acts_as_authentic do |c|
			# DEBUG: only for testing
			# c.logged_in_timeout = 1.minute
		end
	rescue
		# NOTE: ignore errors - only happens when using from a
		# non-fully active record compliant, e.g. from the app installer
	end

	before_create :before_create_hook
	before_save :before_save_hook
	before_destroy :before_destroy_hook 
	after_save :after_save_hook
	after_create :after_create_hook

	def self.system_find_name_by_username(username)
		pwd = StringScanner.new(File.open('/etc/passwd').readlines.join)
		user = Regexp.new("^#{username}:[^:]*:(\\d+):\\d+:([^:]*):", Regexp::MULTILINE)
		pwd.scan_until user or return nil
		uid = pwd[1].to_i
		return nil if uid < 500
		[pwd[2], uid]
	end

	def self.system_all_new_users
		res = []
		Dir.chdir("/home") do
		    Dir.glob("*").sort.reverse.each do | login |
			unless User.find_by_login login
				(name, uid) = system_find_name_by_username login
				res << { :login => login, :name => name } unless name.nil? or name.blank? or uid < 500
			end
		    end
		end
		res
	end

	def self.all_users
		nu = self.system_all_new_users
		unless nu.nil? or nu.empty?
			User.create(nu)
		end
		User.find(:all, :conditions => ["login NOT IN (?)", 'root']).sort { |x,y| x.login <=> y.login }
	end

	def self.system_user_exists? (username)
		system_find_name_by_username(username) != nil
	end

	def self.is_valid_name? (username)
		(name, uid) = system_find_name_by_username(username)
		# only valid names must not exist already
		name == nil
	end

	# User authentication using Linux's PAM
	# Note that the two parameters are plain cleartext strings
	# FIXME - DO NOT USE! this is busted!
	def self.valid_pam_auth?(username, password)
		require 'pam'

		def conversation(msg, data)
			puts "MSG: #{msg.inspec}"
			[PAM::Response.new(password, 0)]
		end

		conv = proc { |msg, data| [PAM::Response.new(password, 0)] }

		PAM.start("amahi", username, conv) do |pam|
			begin
				pam.authenticate(0)
			rescue
				return false
			end
			return true
		end
	end

	# add to the group called "users" so that it's like the rest
	def add_to_users_group
		c = Command.new("usermod -g users -a -G users \"#{self.login}\"")
		c.execute
	end

	def add_or_passwd_change_samba_user
		# adds samba user or simply chages it's password
		pwd_option = password_option()
		c = Command.new("usermod #{pwd_option} \"#{self.login}\"")
		c.execute
		unless self.password.nil? && self.password.blank?
			p = self.password
			c = Command.new "(echo '#{p}'; echo '#{p}') | smbpasswd -s -a \"#{self.login}\""
			c.submit "smbpasswd -e \"#{self.login}\""
			c.execute
		end
	end

	# needs to create authlogic records for this user
	def needs_auth?
		!crypted_password || crypted_password.blank?
	end

	# does this user have admin privileges?
	def admin?
		admin
	end

protected

	def before_create_hook
		# FIXME: this is an issue with fedora 12 and usernames in lowercase
		# https://bugzilla.redhat.com/show_bug.cgi?id=550732
		# http://bugs.amahi.org/issues/show/392
		self.login = self.login.downcase
		return if User.system_user_exists? self.login
		pwd_option = password_option()
		# FIXME: use a different (programmable) group
		c = Command.new "useradd -g users -c \"#{self.name}\" #{pwd_option} \"#{self.login}\""
		# FIXME - we should use add_or_passwd_change_samba_user above! DRY
		unless self.password.nil? && self.password.blank?
			p = self.password
			c.submit("(echo '#{p}'; echo '#{p}') | smbpasswd -s -a \"#{self.login}\"")
			c.submit("smbpasswd -e \"#{self.login}\"")
		end
		c.execute
	end

	# provide a password option with a crypted password suitable for the system
	# NOTE: it's different than the standard crypted password and salt in the user model!
	def password_option
		return "" if self.password.nil? || self.password.blank?
		salt = ('a'..'z').to_a + ('A'..'Z').to_a + ('0'..'9').to_a + ['.', '/']
		salt = (salt.sort_by{rand}.join)[0,2]
		sys_crypted_password = password.crypt(salt)
		"-p \"#{sys_crypted_password}\""
	end

	def before_save_hook
		return unless User.system_user_exists? self.login
		pwd_option = password_option()
		c = Command.new("usermod -c \"#{self.name}\" #{pwd_option} \"#{self.login}\"")
		c.execute
	end

	def after_save_hook
		if admin_changed?
			Share.push_shares
		end
	end

	def after_create_hook
		Share.create_logon_script(self.login)
	end

	def before_destroy_hook
		c = Command.new("smbpasswd -x \"#{self.login}\"")
		c.submit("userdel -r \"#{self.login}\"")
		c.execute
	end
end
