# Amahi Home Server
# Copyright (C) 2007-2011 Amahi
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License v3
# (29 June 2007), as published in the COPYING file.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# file COPYING for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Amahi
# team at http://www.amahi.org/ under "Contact Us."


class Platform

	PLATFORMS=['fedora', 'ubuntu', 'debian', 'mac']
	SERVICES={
		'fedora' => {
			:apache => 'httpd',
			:dhcp => 'dhcpd',
			:dns => 'named',
			:smb => 'smb',
			:nmb => 'nmb',
		},
		'ubuntu' => {
			:apache => 'apache2',
			:dhcp => 'isc-dhcp-server',
			:dns => 'bind9',
			:smb => 'smbd',
			:nmb => 'nmbd',
		},
		'debian' => {
			:apache => 'apache2',
			:dhcp => 'isc-dhcp-server',
			:dns => 'bind9',
			:smb => 'samba',
			:nmb => 'samba',
		},
		'mac' => {
			:apache => 'apache2',
			:dhcp => 'notsure', # FIXME
			:dns => 'notsure', # FIXME
			:smb => 'smbd',
			:nmb => 'nmbd',
		}
	}

	FILENAMES={
		'fedora' => {
			:apache2pid => 'httpd/httpd.pid',
			:dhcpleasefile => '/var/lib/dhcpd/dhcpd.leases',
		},
		'ubuntu' => {
			:apache2pid => 'apache2.pid',
			:dhcpleasefile => '/var/lib/dhcp3/dhcpd.leases',
		},
		'debian' => {
			:apache2pid => 'apache2.pid',
			:dhcpleasefile => '/var/lib/dhcp/dhcpd.leases',
		},
		'mac' => {
			:apache2pid => 'apache2.pid', # not sure; best guess; FIXME
			:dhcpleasefile => 'notsure', # FIXME
		}
	}

	def self.reload(service)
		c = Command.new("sleep 1")
		c.submit("service #{service2name service} reload")
		c.submit("sleep 1")
		c.execute
	end

	def self.file_name(service)
		file2name(service)
	end

	def self.service_name(service)
		service2name(service)
	end

	def self.platform
		@@platform
	end

	def self.fedora?
		@@platform == 'fedora'
	end

	def self.ubuntu?
		@@platform == 'ubuntu'
	end

	def self.debian?
		@@platform == 'debian'
	end

	def self.install(pkgs)
		pkginstall(pkgs)
	end

	def self.uninstall(pkgs)
		pkguninstall(pkgs)
	end

private

	def self.set_platform
		if File.exist?('/etc/issue')
			line = nil
			File.open("/etc/issue", "r") do |issue|
				line = issue.gets
			end
			@@platform = "debian" if line.include?("Debian")
			@@platform = "ubuntu" if line.include?("Ubuntu")
			@@platform = "fedora" if line.include?("Fedora")
		elsif File.exist?('/mach_kernel')
			@@platform = "mac"
		else
			@@platform = nil
		end
		raise "unsupported platform #{@@platform}" unless PLATFORMS.include?(@@platform)
	end

	def self.service2name(service)
		name = SERVICES[@@platform][service]
		raise "unknown service '#{service}' for '#{@@platform}'" unless name
		name
	end

	def self.file2name(fname)
		name = FILENAMES[@@platform][fname]
		raise "unknown filename '#{fname}' for '#{@@platform}'" unless name
		name
	end

	def self.pkginstall(pkgs)
		if debian? or ubuntu?
	                c = Command.new "apt-get -y install #{pkgs}"
			c.run_now
		elsif fedora?
			if pkgs =~ /^\w+:\/\//
				# downloadable RPM
				# FIXME - check the sha1sum
				fname = download_and_check_sha1(pkgs, installer.source_sha1)
				cmd = "rpm -Uvh #{fname}"
			else
				cmd = "yum -y install #{pkgs}"
			end
			c = Command.new cmd
			c.run_now

		else
			raise "unsupported platform #{@@platform}" unless PLATFORMS.include?(@@platform)
		end
	end

	def self.pkguninstall(pkgs)
		if debian? or ubuntu?
	                c = Command.new "apt-get -y remove #{pkgs}"
			c.run_now
		elsif fedora?
			if pkgs =~ /^\w+:\/\/.*\/([^\/]*)\.rpm/
				cmd = "rpm -e #{$1}"
			else
				cmd = "rpm -e #{deps}"
			end
			c = Command.new cmd
			c.run_now
		else
			raise "unsupported platform #{@@platform}" unless PLATFORMS.include?(@@platform)
		end
	end

	# class initialization
	set_platform

end
